from evalassist.const import DATABASE_URL
from sqlmodel import SQLModel, create_engine

from .model import AppUser, LogRecord, StoredTestCase  # noqa: F401

engine = create_engine(DATABASE_URL)
SQLModel.metadata.create_all(engine)



from datetime import datetime
from typing import List, Optional

from sqlmodel import Field, Relationship, SQLModel


class AppUser(SQLModel, table=True):
    __tablename__ = "app_user"
    id: Optional[int] = Field(default=None, primary_key=True)
    email: str = Field(index=True, unique=True)
    name: str
    created_at: datetime = Field(default_factory=datetime.utcnow)
    stored_test_cases: List["StoredTestCase"] = Relationship(back_populates="app_user")


class StoredTestCase(SQLModel, table=True):
    __tablename__ = "stored_test_case"
    id: Optional[int] = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="app_user.id")
    content: str
    name: str

    app_user: Optional[AppUser] = Relationship(back_populates="stored_test_cases")


class LogRecord(SQLModel, table=True):
    __tablename__ = "log_record"
    id: Optional[int] = Field(default=None, primary_key=True)
    data: str



import json
import time
from collections.abc import Callable

from evalassist.model import LogRecord
from fastapi import Request, Response
from fastapi.routing import APIRoute
from sqlmodel import Session
from starlette.background import BackgroundTask

from .database import engine  # Assumes you have engine/session setup

ignored_endpoints = [
    "/health",
    "/evaluators/",
    "/criterias/",
    "/test_case/",
    "/user/",
    "/default-credentials/",
    "/benchmarks//domains-and-personas/",
]


def log_info(method, path, req_body, res_body, headers, runtime):
    record = {
        "path": path,
        "method": method,
        "timestamp": time.time(),
        "runtime": runtime,
    }

    if path in ignored_endpoints:
        return

    if req_body:
        req = json.loads(req_body.decode())
        if "llm_provider_credentials" in req:
            req["llm_provider_credentials"] = ""
        record["request"] = req

    if res_body:
        res = json.loads(res_body.decode())
        record["response"] = res

    if "user_id" in headers:
        record["user_id"] = int(headers.get("user_id"))

    log_record = LogRecord(data=json.dumps(record))
    with Session(engine) as session:
        session.add(log_record)
        session.commit()


class LoggingRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            req_body = await request.body()
            start_timestamp = time.time()
            response = await original_route_handler(request)
            end_timestamp = time.time()
            runtime = round(end_timestamp - start_timestamp, 2)
            tasks = response.background
            request.headers
            task = BackgroundTask(
                log_info,
                request.method,
                request.url.path,
                req_body,
                response.body if hasattr(response, "body") else None,
                request.headers,
                runtime,
            )

            # check if the original response had background tasks already attached to it
            if tasks:
                tasks.add_task(task)  # add the new task to the tasks list
                response.background = tasks
            else:
                response.background = task

            return response

        return custom_route_handler



import click
import uvicorn


@click.group()
def cli():
    pass


@cli.command()
def serve():
    uvicorn.run("evalassist.main:app", host="127.0.0.1", port=8000)


@cli.command()
def version():
    click.echo("EvalAssist v0.1.5")


def main():
    cli()


if __name__ == "__main__":
    cli()



import functools
import json
import logging
import os
import re
import time
import traceback
from enum import Enum
from typing import Optional

from botocore.exceptions import NoCredentialsError
from fastapi import HTTPException
from ibm_watsonx_ai.wml_client_error import (
    ApiRequestFailure,
    CannotSetProjectOrSpace,
    WMLClientError,
)
from langchain_core.exceptions import OutputParserException
from openai import AuthenticationError, NotFoundError
from torch import device
from unitxt.inference import (
    CrossProviderInferenceEngine,
    HFAutoModelInferenceEngine,
    TorchDeviceMixin,
    WMLInferenceEngineGeneration,
)
from unitxt.llm_as_judge import EvaluatorNameEnum, ModelProviderEnum

from .const import (
    EXTENDED_EVALUATOR_TO_MODEL_ID,
    EXTENDED_EVALUATORS_METADATA,
    ExtendedEvaluatorMetadata,
    ExtendedEvaluatorNameEnum,
    ExtendedModelProviderEnum,
)

logger = logging.getLogger(__name__)


def fill_unknown_template(template: str, value: str) -> str:
    # Case 1: Unnamed placeholder(s) like "hello {}"
    if "{}" in template:
        return template.format(value)

    # Case 2: Named placeholder like "hello {world}"
    match = re.search(r"\{(\w+)\}", template)
    if match:
        var_name = match.group(1)
        return template.format(**{var_name: value})

    # Case 3: No placeholder
    return template


def get_custom_models():
    base_path = os.path.dirname(os.path.abspath(__file__))
    json_path = os.path.join(base_path, "..", "custom_models.json")
    if not os.path.exists(json_path):
        json_path = os.path.join(base_path, "..", "..", "custom_models.json")
    if os.path.exists(json_path):
        with open(json_path, "r", encoding="utf-8") as file:
            try:
                custom_models = json.load(file)
                return custom_models
            except Exception:
                raise ValueError("The custom_models json format is wrong")
    else:
        return []


def convert_model_name_wx_to_hf(wx_model_name):
    model_map = {
        "ibm/granite-guardian-3-2b": "ibm-granite/granite-guardian-3.1-2b",
        "ibm/granite-guardian-3-8b": "ibm-granite/granite-guardian-3.1-8b",
    }
    try:
        return model_map[wx_model_name]
    except KeyError:
        return wx_model_name


def get_enum_by_value(value: str, enum: Enum) -> Enum:
    for enum_member in enum:
        if enum_member.value == value:
            return enum_member
    return None


def get_local_hf_inference_engine_params(
    model_name: str,
):
    return {
        "model_name": convert_model_name_wx_to_hf(model_name),
        "max_new_tokens": 1024,
        "device": get_default_torch_device(),
    }


def get_cross_inference_engine_params(
    credentials: dict,
    provider: ModelProviderEnum,
    model_name: str,
    custom_params: dict = None,
    provider_specific_params: dict = None,
):
    provider_specific_args = {}
    inference_engine_params = {
        "max_tokens": 1024,
        "seed": 42,
        "credentials": credentials.copy(),
    }
    if provider == ModelProviderEnum.WATSONX:
        provider_specific_args[ModelProviderEnum.WATSONX] = {
            "max_requests_per_second": 7
        }

    if (
        provider == ModelProviderEnum.AZURE
        or provider == ExtendedModelProviderEnum.OPENAI_LIKE
    ):
        inference_engine_params["credentials"]["api_base"] = fill_unknown_template(
            credentials["api_base"], model_name
        )

    elif provider == ExtendedModelProviderEnum.OPENAI_LIKE:
        provider = ModelProviderEnum.OPENAI

    if custom_params is not None:
        inference_engine_params.update(custom_params)
    if provider_specific_params is not None:
        provider_specific_args.update(provider_specific_params)
    inference_engine_params["model"] = model_name
    inference_engine_params["provider"] = provider.value
    inference_engine_params["provider_specific_args"] = provider_specific_args
    return inference_engine_params


def get_cross_inference_engine(
    credentials: dict[str, str],
    provider: ModelProviderEnum,
    model_name: EvaluatorNameEnum,
    custom_params: dict = None,
    provider_specific_params: dict = None,
):
    inference_engine_params = get_cross_inference_engine_params(
        credentials=credentials,
        provider=provider,
        model_name=model_name,
        custom_params=custom_params,
        provider_specific_params=provider_specific_params,
    )

    return CrossProviderInferenceEngine(**inference_engine_params)


def get_watsonx_inference_engine(
    credentials: dict[str, str],
    provider: ModelProviderEnum,
    model_name: EvaluatorNameEnum,
    custom_params: dict = None,
):
    inference_engine_params = {
        "max_new_tokens": 1024,
        "credentials": {
            "api_key": credentials["api_key"],
            "project_id": credentials["project_id"],
            "url": credentials["api_base"],
        },
        "model_name": model_name,
    }
    if custom_params is not None:
        inference_engine_params.update(custom_params)
    return WMLInferenceEngineGeneration(**inference_engine_params)


preloaded_hf_models = {}


def get_hf_inference_engine(
    model_name: str,
    custom_params: dict = None,
):
    global preloaded_hf_models
    if model_name in preloaded_hf_models:
        logger.debug(f"Using preloaded HF model {model_name}")
        return preloaded_hf_models[model_name]
    else:
        logger.debug(f"Loading model {model_name}")
        params = get_local_hf_inference_engine_params(model_name)
        if custom_params is not None:
            params.update(custom_params)
        hf_model = HFAutoModelInferenceEngine(**params)
        preloaded_hf_models[model_name] = hf_model
        return hf_model


def get_inference_engine(
    credentials: dict[str, str],
    provider: ModelProviderEnum | ExtendedModelProviderEnum,
    model_name: str,
    custom_params: dict = None,
    provider_specific_params: dict = None,
):
    if provider == ExtendedModelProviderEnum.LOCAL_HF:
        return get_hf_inference_engine(model_name, custom_params)
    if provider == ModelProviderEnum.WATSONX and "granite-guardian" in model_name:
        return get_watsonx_inference_engine(
            credentials, provider, model_name, custom_params
        )
    return get_cross_inference_engine(
        credentials, provider, model_name, custom_params, provider_specific_params
    )


def get_model_name_from_evaluator(
    evaluator_metadata: ExtendedEvaluatorMetadata,
    provider: str,
) -> str:
    model_name = EXTENDED_EVALUATOR_TO_MODEL_ID.get(evaluator_metadata.name, None)
    return (
        evaluator_metadata.custom_model_path
        if evaluator_metadata.custom_model_path is not None
        else model_name
    )


def get_evaluator_metadata_wrapper(
    evaluator_name: EvaluatorNameEnum | ExtendedEvaluatorNameEnum,
    custom_model_name: Optional[str] = None,
) -> ExtendedEvaluatorMetadata:
    if evaluator_name.name != ExtendedEvaluatorNameEnum.CUSTOM.name:
        evaluator_search = [
            e
            for e in EXTENDED_EVALUATORS_METADATA
            if e.name.name == evaluator_name.name
        ]
        if len(evaluator_search) == 0:
            raise ValueError(f"An evaluator with id {evaluator_name} does not exist.")
        if len(evaluator_search) > 1:
            raise ValueError(
                f"An evaluator with id {evaluator_name} matched several models."
            )
        return evaluator_search[0]
    else:
        custom_models = get_custom_models()
        if custom_model_name not in [
            custom_model["name"] for custom_model in custom_models
        ]:
            raise ValueError("The specified custom model was not found")

        custom_model = [
            custom_model
            for custom_model in custom_models
            if custom_model["name"] == custom_model_name
        ][0]
        return ExtendedEvaluatorMetadata(
            name=evaluator_name,
            custom_model_name=custom_model["name"],
            custom_model_path=custom_model["path"],
            providers=[
                ExtendedModelProviderEnum(p)
                if p in ExtendedModelProviderEnum
                else ModelProviderEnum(p)
                for p in custom_model["providers"]
            ],
        )


def get_default_torch_device(avoid_mps: bool = False) -> device:
    device = TorchDeviceMixin().get_device_id()
    logger.debug(f"Detected device: {device}")
    return device


def init_evaluator_name(
    evaluator_name: str,
) -> tuple[ExtendedEvaluatorNameEnum, Optional[str]]:
    evaluator_name_as_enum = get_enum_by_value(evaluator_name, EvaluatorNameEnum)
    if evaluator_name_as_enum is None:
        evaluator_name_as_enum = get_enum_by_value(
            evaluator_name, ExtendedEvaluatorNameEnum
        )
    custom_model_name = None
    if evaluator_name_as_enum is not None:
        return (evaluator_name_as_enum, None)

    custom_model_name = evaluator_name
    evaluator_name = ExtendedEvaluatorNameEnum.CUSTOM
    return (evaluator_name, custom_model_name)


def log_runtime(function):
    """
    Usage: wrap a function call with the log_runtime function to log its runtime
    """

    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        start_time = time.time()

        res = function(*args, **kwargs)

        end_time = time.time()
        total_time = round(end_time - start_time, 2)

        logger.debug(
            f"{function.__name__} took {total_time} seconds, {round(total_time / 60, 2)} minutes"
        )

        return res

    return wrapper


def to_snake_case(name: str) -> str:
    return name.replace(" ", "_").lower()


def parse_exception_message(e: Exception) -> str:
    error_message = e.message if hasattr(e, "message") else None
    if error_message is None:
        return str(e)
    result = error_message.split(" - ", 1)
    if len(result) == 1:
        return result[0]
    try:
        result = json.loads(result[1])
    except json.JSONDecodeError:
        return result[1]
    if "errorMessage" in result:
        return result["errorMessage"]
    elif (
        "errors" in result
        and len(result["errors"]) > 0
        and "message" in result["errors"][0]
    ):
        return result["errors"][0]["message"]
    elif "title" in result:
        return result["detail"]
    else:
        return str(result)


def handle_exception(e: Exception) -> None:
    error_message = parse_exception_message(e)
    if isinstance(e, OutputParserException):
        raise HTTPException(
            status_code=400,
            detail=error_message,
        ) from e
    elif isinstance(e, NotFoundError):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, ValueError):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, ApiRequestFailure):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, NoCredentialsError):
        raise HTTPException(
            status_code=400,
            detail=error_message,
        )
    elif isinstance(e, AuthenticationError):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, CannotSetProjectOrSpace):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, WMLClientError):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, AssertionError):
        raise HTTPException(status_code=400, detail=error_message)
    elif isinstance(e, RuntimeError):
        if e.__cause__:
            handle_exception(e.__cause__)
        else:
            raise HTTPException(400, error_message)
    else:
        raise HTTPException(
            status_code=400,
            detail=error_message,
        )


def handle_llm_generation_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            handle_exception(e)
        finally:
            traceback.print_exc()

    return wrapper


def clean_object(results: dict | list):
    """Cleans the object by removing `None` values and empty lists and dictionaries.

    Args:
        results (Union[dict, list]): The results to clean.

    Returns:
        Union[dict, list]: The cleaned results.
    """
    if isinstance(results, list):
        return [clean_object(x) for x in results]
    cleaned = {
        k: (v if not isinstance(v, dict) else clean_object(v))
        for k, v in results.items()
        if v is not None and not (isinstance(v, (list, dict)) and len(v) == 0)
    }
    # Remove the dictionary itself if it becomes empty
    return {
        k: v for k, v in cleaned.items() if not (isinstance(v, dict) and len(v) == 0)
    }



from abc import ABC
from typing import Any, List, Optional, cast

from unitxt.api import evaluate, load_dataset
from unitxt.blocks import Task, TaskCard
from unitxt.llm_as_judge import (
    Criteria,
    CriteriaWithOptions,
    EvaluatorNameEnum,
    EvaluatorTypeEnum,
    LLMJudgeDirect,
    LLMJudgePairwise,
    LoadCriteria,
    LoadCriteriaWithOptions,
    ModelProviderEnum,
)
from unitxt.loaders import LoadFromDictionary
from unitxt.metrics import RISK_TYPE_TO_CLASS, GraniteGuardianBase, RiskType
from unitxt.templates import NullTemplate

from ..api.common import (
    DirectPositionalBias,
    DirectResultModel,
    Instance,
    PairwiseResultModel,
)
from ..const import ExtendedEvaluatorNameEnum, ExtendedModelProviderEnum
from ..utils import (
    get_evaluator_metadata_wrapper,
    get_inference_engine,
    get_model_name_from_evaluator,
)


class Evaluator(ABC):
    evaluator_type: EvaluatorTypeEnum

    def __init__(
        self,
        evaluator_name: EvaluatorNameEnum | ExtendedEvaluatorNameEnum,
        custom_model_name: Optional[str] = None,
    ):
        self.evaluator_metadata = get_evaluator_metadata_wrapper(
            evaluator_name, custom_model_name
        )

    def get_preprocess_steps(self):
        raise NotImplementedError("This method must be implemented.")

    def parse_results(self, dataset):
        raise NotImplementedError("This method must be implemented.")

    def get_prediction_type(self):
        raise NotImplementedError("This method must be implemented.")

    def get_evaluator_klass(self):
        raise NotImplementedError("This method must be implemented.")

    def get_predictions(self, instances: list[Instance]) -> list[str | list[str]]:
        return [instance.response for instance in instances]

    def evaluate(
        self,
        instances: list[Instance],
        criteria: Criteria | CriteriaWithOptions,
        provider: ModelProviderEnum | ExtendedModelProviderEnum,
        credentials: dict[str, str],
    ):
        model_name = get_model_name_from_evaluator(
            self.evaluator_metadata,
            provider,
        )

        inference_engine = get_inference_engine(
            credentials,
            provider,
            model_name,
            custom_params={
                "use_cache": True,
                "seed": 42,
                "temperature": 0,
            },
        )

        context_variables_list = [instance.context_variables for instance in instances]

        evalutor_params = {
            "inference_engine": inference_engine,
            "context_fields": list(context_variables_list[0].keys()),
            "criteria_field": "criteria",
            "generate_summaries": False,
        }

        input_fields = {
            input_field: str for input_field in context_variables_list[0].keys()
        }
        metric = self.get_evaluator_klass()(**evalutor_params)
        data = {"test": context_variables_list}
        data["test"] = [{**d, "judgement": criteria} for d in data["test"]]
        card = TaskCard(
            loader=LoadFromDictionary(data=data, data_classification_policy=["public"]),
            preprocess_steps=self.get_preprocess_steps(),
            task=Task(
                input_fields=input_fields,
                prediction_type=self.get_prediction_type(),
                metrics=[metric],
                reference_fields={"criteria": Any},
                default_template=NullTemplate(),
            ),
        )

        dataset = load_dataset(card=card, split="test")
        predictions = self.get_predictions(instances)
        evaluated_dataset = evaluate(predictions=predictions, data=dataset)
        per_instance_result = self.parse_results(evaluated_dataset)
        results = []
        for instance_result, instance in zip(per_instance_result, instances):
            results.append({"id": instance.id, "result": instance_result})
        return results


class DirectAssessmentEvaluator(Evaluator):
    def __init__(
        self,
        evaluator_name: EvaluatorNameEnum | ExtendedEvaluatorNameEnum,
        custom_model_name: Optional[str] = None,
    ):
        super().__init__(evaluator_name, custom_model_name)
        self.evaluator_type = EvaluatorTypeEnum.DIRECT

    def get_preprocess_steps(self):
        return [LoadCriteriaWithOptions(field="judgement", to_field="criteria")]

    def get_prediction_type(self):
        return str

    def get_evaluator_klass(self):
        return LLMJudgeDirect

    def parse_results(self, dataset) -> list[DirectResultModel]:
        results = []
        prefix = dataset[0]["score"]["instance"]["score_name"]
        for instance in dataset:
            instance_score = instance["score"]["instance"]
            positional_bias = DirectPositionalBias(
                detected=instance_score[f"{prefix}_positional_bias"],
            )
            if positional_bias.detected:
                positional_bias.option = instance_score[
                    f"{prefix}_positional_bias_selected_option"
                ]
                positional_bias.explanation = instance_score[
                    f"{prefix}_positional_bias_assessment"
                ]

            results.append(
                DirectResultModel(
                    option=instance_score[f"{prefix}_selected_option"],
                    explanation=instance_score[f"{prefix}_assessment"],
                    positional_bias=positional_bias,
                )
            )
        return results


class PairwiseComparisonEvaluator(Evaluator):
    def __init__(
        self,
        evaluator_name: EvaluatorNameEnum | ExtendedEvaluatorNameEnum,
        custom_model_name: Optional[str] = None,
    ):
        super().__init__(evaluator_name, custom_model_name)
        self.evaluator_type = EvaluatorTypeEnum.PAIRWISE

    def get_preprocess_steps(self):
        return [LoadCriteria(field="judgement", to_field="criteria")]

    def get_prediction_type(self):
        return List[str]

    def get_evaluator_klass(self):
        return LLMJudgePairwise

    def parse_results(self, dataset):
        results = []
        for instance in dataset:
            score = instance["score"]["instance"]

            parsed_score = {}
            for key in score.keys():
                outer_key = key.split("_")[0]
                if outer_key not in ["score", "criteria"]:
                    parsed_score[outer_key] = PairwiseResultModel(
                        **{
                            "contest_results": score[f"{outer_key}_contest_results"],
                            "compared_to": score[f"{outer_key}_compared_to"],
                            "explanations": score[f"{outer_key}_assessments"],
                            "positional_bias": score[f"{outer_key}_positional_bias"],
                            "winrate": score[f"{outer_key}_winrate"],
                            "ranking": score[f"{outer_key}_ranking"],
                            "selections": score[f"{outer_key}_selections"],
                        }
                    )
            results.append(parsed_score)

        return results


class GraniteGuardianEvaluator(ABC):
    evaluator_type: EvaluatorTypeEnum = EvaluatorTypeEnum.DIRECT
    field_map = {
        "user_message_field": "user_message",
        "assistant_message_field": "assistant_message",
        "context_field": "context",
        "tools_field": "tools",
    }

    def __init__(self, name: ExtendedEvaluatorNameEnum):
        self.evaluator_name = name

    def get_harms_and_risks_result_description(
        self, evaluated_component, criteria_name
    ):
        messages = {
            "general_harm": f"Is the {evaluated_component} harmful by common-sense?",
            "social_bias": f"Is the {evaluated_component} prejudiced based on identity or characteristics?",
            "jailbreak": f"Does the {evaluated_component} try to manipulate the AI to generate harmful, undesired, or inappropriate content?",
            "violence": f"Does the {evaluated_component} contain content promoting physical, mental, or sexual harm?",
            "profanity": f"Does the {evaluated_component} include offensive language or insults?",
            "unethical_behavior": f"Does the {evaluated_component} include actions that violate moral or legal standards?",
            "answer_relevance": f"Does the {evaluated_component} fail to address or properly answer the user question?",
            "context_relevance": "Is the retrieved context irrelevant to the user question or does not address their needs?",
            "groundedness": f"Does the {evaluated_component} include claims or facts not supported by or contradicted by the provided context?",
        }
        if criteria_name not in messages:
            return f"Is the {evaluated_component} harmful based on the risk definition?"

        return messages[criteria_name]

    def get_predictions(self, instances: list[Instance]) -> list[str | list[str]]:
        return [instance.response for instance in instances]

    def get_prompt(self, risk_name, instances) -> str:
        risk_name = self.get_risk_name(risk_name)
        predictions = self.get_predictions(instances)

        context_variables_list = self.get_context_variables(
            instances=instances, predictions=predictions
        )
        input_fields = self.get_input_fields(context_variables_list)
        granite_guardian_class = self.getEvaluatorClass(
            self.infer_risk_type(
                risk_name=risk_name, field_map=self.field_map, input_fields=input_fields
            )
        )

        metric = cast(
            GraniteGuardianBase,
            granite_guardian_class(
                risk_name=risk_name,
                **self.field_map,
            ),
        )

        return [
            metric.get_prompt(metric.process_input_fields(task_data=context_variables))
            for context_variables in context_variables_list
        ]

    def parse_results(
        self, dataset, response_variable_name: str
    ) -> list[DirectResultModel]:
        results = []
        for instance in dataset:
            risk_name: str = instance["score"]["instance"]["score_name"]
            instance_score = instance["score"]["instance"]
            explanation = self.get_harms_and_risks_result_description(
                response_variable_name.replace("_", " "),
                risk_name.lower().replace(" ", "_"),
            )

            instance_score = instance["score"]["instance"]
            positional_bias = DirectPositionalBias(
                detected=False, option="", explanation=""
            )
            selected_option = instance_score[f"{risk_name}_label"]

            if selected_option is None:
                raise ValueError("Granite Guardian evaluation failed")

            results.append(
                DirectResultModel(
                    option=instance_score[f"{risk_name}_label"],
                    explanation=explanation,
                    positional_bias=positional_bias,
                )
            )
        return results

    def get_risk_name(self, unparsed_risk_name: str):
        risk_name = unparsed_risk_name
        risk_name = (
            "_".join(
                risk_name.split(" ")[(1 if risk_name.startswith("Context") else 2) :]
            )
            .lower()
            .replace(" ", "_")
        )

        return risk_name if risk_name != "general_harm" else "harm"

    def get_context_variables(
        self, instances: list[Instance], predictions: list[str]
    ) -> list[dict[str, str]]:
        context_variables_list = [instance.context_variables for instance in instances]
        for context_variables, prediction in zip(context_variables_list, predictions):
            # use prediction as one more context variable
            context_variables[instances[0].response_variable_name] = prediction

        return [
            {k.lower().replace(" ", "_"): v for k, v in context_variables.items()}
            for context_variables in context_variables_list
        ]

    def get_input_fields(self, context_variables_list: list[dict[str, str]]):
        return {input_field: str for input_field in context_variables_list[0].keys()}

    def evaluate(
        self,
        criteria: Criteria | CriteriaWithOptions,
        provider: ModelProviderEnum,
        credentials: dict[str, str],
        instances: list[Instance],
    ):
        risk_name = self.get_risk_name(criteria.name)
        predictions = self.get_predictions(instances)
        context_variables_list = self.get_context_variables(
            instances=instances, predictions=predictions
        )
        input_fields = self.get_input_fields(context_variables_list)

        granite_guardian_class = self.getEvaluatorClass(
            self.infer_risk_type(
                risk_name=risk_name, field_map=self.field_map, input_fields=input_fields
            )
        )

        custom_params = {}
        self.evaluator_metadata = get_evaluator_metadata_wrapper(self.evaluator_name)
        model_name = get_model_name_from_evaluator(
            self.evaluator_metadata,
            provider,
        )
        inference_engine = get_inference_engine(
            credentials, provider, model_name, custom_params
        )

        metric = granite_guardian_class(
            risk_name=risk_name,
            **self.field_map,
            inference_engine=inference_engine,
        )
        data = {"test": context_variables_list}

        card = TaskCard(
            loader=LoadFromDictionary(data=data, data_classification_policy=["public"]),
            task=Task(
                input_fields=input_fields,
                reference_fields={},
                prediction_type=float,
                metrics=[metric],
                default_template=NullTemplate(),
            ),
        )

        dataset = load_dataset(card=card, split="test")
        evaluated_dataset = evaluate(predictions=[0.0 for _ in dataset], data=dataset)

        per_instance_result = self.parse_results(
            evaluated_dataset, instances[0].response_variable_name
        )
        results = []
        for instance_result, instance in zip(per_instance_result, instances):
            results.append({"id": instance.id, "result": instance_result})
        return results

    def infer_risk_type(
        self, risk_name: str, field_map: dict[str, str], input_fields: dict[str, str]
    ) -> RiskType:
        """
        Infers the RiskType based on the risk_name and the provided input fields keys.
        """

        available_risks = GraniteGuardianBase.available_risks

        if risk_name in available_risks[RiskType.ASSISTANT_MESSAGE]:
            if field_map["assistant_message_field"] in input_fields:
                return RiskType.ASSISTANT_MESSAGE
            return RiskType.USER_MESSAGE

        if risk_name in available_risks[RiskType.USER_MESSAGE]:
            return RiskType.USER_MESSAGE

        if risk_name in available_risks[RiskType.RAG]:
            return RiskType.RAG

        if risk_name in available_risks[RiskType.AGENTIC]:
            return RiskType.AGENTIC

        return RiskType.CUSTOM_RISK

    def getEvaluatorClass(self, risk_type: RiskType) -> GraniteGuardianBase:
        return RISK_TYPE_TO_CLASS[risk_type]
